package istia.st.springmvc.personnes.dao;

import istia.st.springmvc.personnes.entites.Personne;
import org.springframework.orm.ibatis.support.SqlMapClientDaoSupport;

import java.util.Collection;

public class DaoImplCommon extends SqlMapClientDaoSupport implements
		IDao {

	// liste des personnes
	public Collection getAll() {
		return getSqlMapClientTemplate().queryForList("Personne.getAll", null);
	}

	// obtenir une personne en particulier
	public Personne getOne(int id) {
		// on la rcupre dans la BD
		Personne personne = (Personne) getSqlMapClientTemplate()
				.queryForObject("Personne.getOne", new Integer(id));
		// a-t-on rcupr qq chose ?
		if (personne == null) {
			// on lance une exception
			throw new DaoException(
					"La personne d'id [" + id + "] n'existe pas", 2);
		}
		// on rend la personne
		return personne;
	}

	// suppression d'une personne
	public void deleteOne(int id) {
		// on supprime la personne
		int n = getSqlMapClientTemplate().delete("Personne.deleteOne",
				new Integer(id));
		// a-t-on russi
		if (n == 0) {
			throw new DaoException("Personne d'id [" + id + "] inconnue", 2);
		}
	}

	// ajouter ou modifier une personne
	public void saveOne(Personne personne) {
		// le paramtre personne est-il valide ?
		check(personne);
		// ajout ou modification ?
		if (personne.getId() == -1) {
			// ajout
			insertPersonne(personne);
		} else {
			updatePersonne(personne);
		}
	}

	// ajouter une personne
	protected void insertPersonne(Personne personne) {
		// 1re version
		personne.setVersion(1);
		// on attend 10 ms - pour les tests mettre true au lieu de false
		if (true)
			wait(10);
		// on insre la nouvelle personne dans la table de la BD
		getSqlMapClientTemplate().insert("Personne.insertOne", personne);
	}

	// modifier une personne
	protected void updatePersonne(Personne personne) {
		// on attend 10 ms - pour les tests mettre true au lieu de false
		if (true)
			wait(10);
		// modification
		// on modifie la personne qui a la bonne version
		int n = getSqlMapClientTemplate()
				.update("Personne.updateOne", personne);
		if (n == 0)
			throw new DaoException("La personne d'Id [" + personne.getId()
					+ "] n'existe pas ou bien a t modifie", 2);
	}

	// vrification validit d'une personne
	private void check(Personne p) {
		// personne p
		if (p == null) {
			throw new DaoException("Personne null", 10);
		}
		// id
		if (p.getId() != -1 && p.getId() < 0) {
			throw new DaoException("Id [" + p.getId() + "] invalide", 11);
		}
		// date de naissance
		if (p.getDateNaissance() == null) {
			throw new DaoException("Date de naissance manquante", 12);
		}
		// nombre d'enfants
		if (p.getNbEnfants() < 0) {
			throw new DaoException("Nombre d'enfants [" + p.getNbEnfants()
					+ "] invalide", 13);
		}
		// nom
		if (p.getNom() == null || p.getNom().trim().length() == 0) {
			throw new DaoException("Nom manquant", 14);
		}
		// prnom
		if (p.getPrenom() == null || p.getPrenom().trim().length() == 0) {
			throw new DaoException("Prnom manquant", 15);
		}
	}

	// attente
	private void wait(int N) {
		// on attend N ms
		try {
			Thread.sleep(N);
		} catch (InterruptedException e) {
			// on affiche la trace de l'exception
			e.printStackTrace();
			return;
		}
	}

}